<?php

if (!defined('ABSPATH'))
    exit;

class ACOVSW_Backend
{

    /**
     * @var    object
     * @access  private
     * @since    1.0.0
     */
    private static $_instance = null;

    /**
     * The version number.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_version;

    /**
     * The token.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_token;

    /**
     * The main plugin file.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $file;

    /**
     * The main plugin directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $dir;

    /**
     * The plugin assets directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_dir;

    /**
     * Suffix for Javascripts.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $script_suffix;

    /**
     * The plugin assets URL.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_url;
    public $hook_suffix = array();
    public $plugin_slug;

    /**
     * Constructor function.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function __construct($file = '', $version = '1.0.0')
    {
        $this->_version = $version;
        $this->_token = ACOVSW_TOKEN;
        $this->file = $file;
        $this->dir = dirname($this->file);
        $this->assets_dir = trailingslashit($this->dir) . 'assets';
        $this->assets_url = esc_url(trailingslashit(plugins_url('/assets/', $this->file)));

        $this->plugin_slug = 'abc';

        $this->script_suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';


        register_activation_hook($this->file, array($this, 'install'));
        // register_deactivation_hook($this->file, array($this, 'deactivation'));
        add_action('save_post', array($this, 'delete_transient'), 1);
        add_action('edited_term', array($this, 'delete_transient'));
        add_action('delete_term', array($this, 'delete_transient'));
        add_action('created_term', array($this, 'delete_transient'));

        add_action('admin_menu', array($this, 'register_root_page'));
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'), 10, 1);
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_styles'), 10, 1);

        $plugin = plugin_basename($this->file);
        add_filter("plugin_action_links_$plugin", array($this, 'add_settings_link'));

        // Custom Attributes
        add_filter( 'woocommerce_product_data_tabs', array ( $this,'acovsw_custom_attributes' ) );
        add_action( 'woocommerce_product_data_panels', array ( $this,'acovsw_custom_attributes_settings' ) );
        
    }

    /**
     *
     *
     * Ensures only one instance of WCPA is loaded or can be loaded.
     *
     * @since 1.0.0
     * @static
     * @see WordPress_Plugin_Template()
     * @return Main WCPA instance
     */
    public static function instance($file = '', $version = '1.0.0')
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($file, $version);
        }
        return self::$_instance;
    }

    public function register_root_page()
    {
        $this->hook_suffix[] = add_menu_page(
            __('Variation Swatches', 'aco-variation-swatches-for-woocommerce'), __('Variation Swatches', 'aco-variation-swatches-for-woocommerce'), 'edit_products', 'acovsw-attributes', array($this, 'admin_ui'), esc_url($this->assets_url) . '/images/icon.png', 25);
        $this->hook_suffix[] = add_submenu_page(
            'acovsw-attributes', __('Settings', 'aco-variation-swatches-for-woocommerce'), __('Settings', 'aco-variation-swatches-for-woocommerce'), 'edit_products', 'acovsw-settings', array($this, 'admin_ui_settings'));
    }

    public function add_settings_link($links)
    {
        $settings = '<a href="' . admin_url('admin.php?page=acovsw-attributes') . '">' . __('Variation Swatches','aco-variation-swatches-for-woocommerce') . '</a>';
        array_push($links, $settings);
        return $links;
    }

    /**
     *    Create post type forms
     */

     static function view($view, $data = array())
    {
        extract($data);
        include(plugin_dir_path(__FILE__) . 'views/' . $view . '.php');
    }

    // End admin_enqueue_styles ()

    public function admin_ui()
    {
        ACOVSW_Backend::view('admin-attributes', []);
    }

    public function admin_ui_settings()
    {
        ACOVSW_Backend::view('admin-settings', []);
    }

    /**
     * Load admin CSS.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function admin_enqueue_styles($hook = '')
    {
                
        $currentScreen = get_current_screen();
        $screenID = $currentScreen->id; //
        if ( strpos($screenID, 'acovsw-') !== false || $screenID == 'product' ) {

            wp_register_style($this->_token . '-admin', esc_url($this->assets_url) . 'css/backend.css', array(), $this->_version);
            wp_enqueue_style($this->_token . '-admin');
            
        }
    }

    /**
     * Load admin Javascript.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function admin_enqueue_scripts($hook = '')
    {

        $currentScreen = get_current_screen();
        $screenID = $currentScreen->id; 
        
        if ( strpos($screenID, 'acovsw-') !== false || $screenID == 'product' ) { 

            if (!isset($this->hook_suffix) || empty($this->hook_suffix)) { 
                return;
            }

            // All Categories
            $attributes = wc_get_attribute_taxonomies();
            $attrs = [];
            if ( $attributes ) {
                foreach ( $attributes as $attribute ) {
                    $attr_id = $attribute->attribute_id;
                    $attr_label = $attribute->attribute_label;
                    $attr_name = $attribute->attribute_name;
                    // $attr_tax = 'pa_'.$attr_name;
                    // $options = array ('hide_empty' => false );
                    // $attr = get_terms ( $attr_tax, $options );
                    $attrs[] = array ( 'attr_id' => $attr_id, 'attr_label' => $attr_label, 'attr_name' => $attr_name );
                }
            }

            $screen = get_current_screen();

            wp_enqueue_script('jquery');

            // Preview Box
            $previewImage = plugin_dir_url(__FILE__). '../assets/images/no-image.svg';
            $productID      = '';

            if ( $screenID == 'product' ) {
                global $post;
                $productID = $post->ID;
            }

            if ( in_array($screen->id, $this->hook_suffix) || $screenID == 'product' ) { 

                if (!wp_script_is('wp-i18n', 'registered')) {
                    wp_register_script('wp-i18n', esc_url($this->assets_url) . 'js/i18n.min.js', array('jquery'), $this->_version, true);
                }

                wp_enqueue_script($this->_token . '-backend-script', esc_url($this->assets_url) . 'js/backend.js', array('jquery', 'wp-i18n'), $this->_version, true);
                wp_localize_script($this->_token . '-backend-script', 'acovsw_object', array(
                        'api_nonce' => wp_create_nonce('wp_rest'),
                        'root' => rest_url('acovsw/v1/'),
                        'previewImage' => $previewImage,
                        'attributes' => (array)$attrs,
                        'productID' => $productID
                    )
                );

                $plugin_rel_path = (dirname($this->file)) . '/../../languages'; /* Relative to WP_PLUGIN_DIR */
                if ( ACOVSW_Wordpress_Version >= 5 ) {
                    wp_set_script_translations(ACOVSW_TOKEN . '-backend-script', 'aco-variation-swatches-for-woocommerce', $plugin_rel_path);
                }

            }

        }

    }


    /* 
    * Custom Attributes
    * @since 3.0.0
    */
    public function acovsw_custom_attributes ( $tabs ) { 
        
        $tabs['acovsw-custom-swatches-settings'] = array(
            'label'    => __( 'Swatches', 'aco-variation-swatches-for-woocommerce' ),
            'target'   => 'acovsw-custom-attributes-settings',
            'class'    => array( 'variations_tab', 'show_if_variable', ),
            'priority' => 65,
        );
        return $tabs;

    }

    public function acovsw_custom_attributes_settings () { ?>

        <div id="acovsw-custom-attributes-settings" class="panel wc-metaboxes-wrapper hidden">
            <?php ACOVSW_Backend::view('admin-product', []); ?>
        </div> 
        
    <?php }

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }

    /**
     * Installation. Runs on activation.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function install()
    {
        $this->_log_version_number();

    }

    /**
     * Log the plugin version number.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    private function _log_version_number()
    {
        update_option($this->_token . '_version', $this->_version);
    }

    public function delete_transient($arg = false)
    {
         delete_transient(ACOVSW_PRODUCTS_TRANSIENT_KEY);
    }

}
